@@ -1,20 +1,28 @@
+"""Tests for TCP connection handling, including proper and timely close."""
+
 from cherrypy.test import test
 test.prefer_parent_path()
 
 import httplib
+import urllib
 import socket
 import sys
 import time
-timeout = 0.1
+timeout = 1
 
 
 import cherrypy
 from cherrypy.test import webtest
+from cherrypy import _cperror
 
 
 pov = 'pPeErRsSiIsStTeEnNcCeE oOfF vViIsSiIoOnN'
 
 def setup_server():
+    
+    def raise500():
+        raise cherrypy.HTTPError(500)
+    
     class Root:
         
         def index(self):
@@ -28,6 +36,10 @@ def setup_server():
             return "Hello, world!"
         hello.exposed = True
         
+        def timeout(self, t):
+            return str(cherrypy.server.httpserver.timeout)
+        timeout.exposed = True
+        
         def stream(self, set_cl=False):
             if set_cl:
                 cherrypy.response.headers['Content-Length'] = 10
@@ -40,7 +52,14 @@ def setup_server():
         stream.exposed = True
         stream._cp_config = {'response.stream': True}
         
+        def error(self, code=500):
+            raise cherrypy.HTTPError(code)
+        error.exposed = True
+        
         def upload(self):
+            if not cherrypy.request.method == 'POST':
+                raise AssertionError("'POST' != request.method %r" %
+                                     cherrypy.request.method)
             return ("thanks for '%s' (%s)" %
                     (cherrypy.request.body.read(),
                      cherrypy.request.headers['Content-Type']))
@@ -50,10 +69,19 @@ def setup_server():
             cherrypy.response.status = response_code
             return "Code = %s" % response_code
         custom.exposed = True
+        
+        def err_before_read(self):
+            return "ok"
+        err_before_read.exposed = True
+        err_before_read._cp_config = {'hooks.on_start_resource': raise500}
+
+        def one_megabyte_of_a(self):
+            return ["a" * 1024] * 1024
+        one_megabyte_of_a.exposed = True
     
     cherrypy.tree.mount(Root())
     cherrypy.config.update({
-        'server.max_request_body_size': 100,
+        'server.max_request_body_size': 1001,
         'environment': 'test_suite',
         })
 
@@ -143,6 +171,12 @@ class ConnectionTests(helper.CPWebCase):
                     
                     # Make another request on the same connection, which should error.
                     self.assertRaises(httplib.NotConnected, self.getPage, "/")
+                
+                # Try HEAD. See http://www.cherrypy.org/ticket/864.
+                self.getPage("/stream", method='HEAD')
+                self.assertStatus('200 OK')
+                self.assertBody('')
+                self.assertNoHeader("Transfer-Encoding")
         else:
             self.PROTOCOL = "HTTP/1.0"
             
@@ -184,76 +218,87 @@ class ConnectionTests(helper.CPWebCase):
             print "skipped ",
             return
         
-        old_timeout = None
-        try:
-            httpserver = cherrypy.server.httpservers.keys()[0]
-            old_timeout = httpserver.timeout
-        except (AttributeError, IndexError):
-            print "skipped ",
-            return
+        self.PROTOCOL = "HTTP/1.1"
         
+        # Make an initial request
+        self.persistent = True
+        conn = self.HTTP_CONN
+        conn.putrequest("GET", "/timeout?t=%s" % timeout, skip_host=True)
+        conn.putheader("Host", self.HOST)
+        conn.endheaders()
+        response = conn.response_class(conn.sock, method="GET")
+        response.begin()
+        self.assertEqual(response.status, 200)
+        self.body = response.read()
+        self.assertBody(str(timeout))
+        
+        # Make a second request on the same socket
+        conn._output('GET /hello HTTP/1.1')
+        conn._output("Host: %s" % self.HOST)
+        conn._send_output()
+        response = conn.response_class(conn.sock, method="GET")
+        response.begin()
+        self.assertEqual(response.status, 200)
+        self.body = response.read()
+        self.assertBody("Hello, world!")
+        
+        # Wait for our socket timeout
+        time.sleep(timeout * 2)
+        
+        # Make another request on the same socket, which should error
+        conn._output('GET /hello HTTP/1.1')
+        conn._output("Host: %s" % self.HOST)
+        conn._send_output()
+        response = conn.response_class(conn.sock, method="GET")
         try:
-            httpserver.timeout = timeout
-            self.PROTOCOL = "HTTP/1.1"
-            
-            # Make an initial request
-            self.persistent = True
-            conn = self.HTTP_CONN
-            conn.putrequest("GET", "/", skip_host=True)
-            conn.putheader("Host", self.HOST)
-            conn.endheaders()
-            response = conn.response_class(conn.sock, method="GET")
             response.begin()
-            self.assertEqual(response.status, 200)
-            self.body = response.read()
-            self.assertBody(pov)
-            
-            # Make a second request on the same socket
-            conn._output('GET /hello HTTP/1.1')
-            conn._output("Host: %s" % self.HOST)
-            conn._send_output()
-            response = conn.response_class(conn.sock, method="GET")
-            response.begin()
-            self.assertEqual(response.status, 200)
-            self.body = response.read()
-            self.assertBody("Hello, world!")
-            
-            # Wait for our socket timeout
-            time.sleep(timeout * 2)
-            
-            # Make another request on the same socket, which should error
-            conn._output('GET /hello HTTP/1.1')
-            conn._output("Host: %s" % self.HOST)
-            conn._send_output()
-            response = conn.response_class(conn.sock, method="GET")
-            try:
-                response.begin()
-            except:
-                if not isinstance(sys.exc_info()[1],
-                                  (socket.error, httplib.BadStatusLine)):
-                    self.fail("Writing to timed out socket didn't fail"
-                              " as it should have: %s" % sys.exc_info()[1])
-            else:
+        except:
+            if not isinstance(sys.exc_info()[1],
+                              (socket.error, httplib.BadStatusLine)):
+                self.fail("Writing to timed out socket didn't fail"
+                          " as it should have: %s" % sys.exc_info()[1])
+        else:
+            if response.status != 408:
                 self.fail("Writing to timed out socket didn't fail"
                           " as it should have: %s" %
                           response.read())
-            
-            conn.close()
-            
-            # Make another request on a new socket, which should work
-            self.persistent = True
-            conn = self.HTTP_CONN
-            conn.putrequest("GET", "/", skip_host=True)
-            conn.putheader("Host", self.HOST)
-            conn.endheaders()
-            response = conn.response_class(conn.sock, method="GET")
-            response.begin()
-            self.assertEqual(response.status, 200)
-            self.body = response.read()
-            self.assertBody(pov)
-        finally:
-            if old_timeout is not None:
-                httpserver.timeout = old_timeout
+        
+        conn.close()
+        
+        # Make another request on a new socket, which should work
+        self.persistent = True
+        conn = self.HTTP_CONN
+        conn.putrequest("GET", "/", skip_host=True)
+        conn.putheader("Host", self.HOST)
+        conn.endheaders()
+        response = conn.response_class(conn.sock, method="GET")
+        response.begin()
+        self.assertEqual(response.status, 200)
+        self.body = response.read()
+        self.assertBody(pov)
+        
+        # Make another request on the same socket,
+        # but timeout on the headers
+        conn.send('GET /hello HTTP/1.1')
+        # Wait for our socket timeout
+        time.sleep(timeout * 2)
+        response = conn.response_class(conn.sock, method="GET")
+        response.begin()
+        self.assertEqual(response.status, 408)
+        conn.close()
+        
+        # Retry the request on a new connection, which should work
+        self.persistent = True
+        conn = self.HTTP_CONN
+        conn.putrequest("GET", "/", skip_host=True)
+        conn.putheader("Host", self.HOST)
+        conn.endheaders()
+        response = conn.response_class(conn.sock, method="GET")
+        response.begin()
+        self.assertEqual(response.status, 200)
+        self.body = response.read()
+        self.assertBody(pov)
+        conn.close()
     
     def test_HTTP11_pipelining(self):
         if cherrypy.server.protocol_version != "HTTP/1.1":
@@ -343,6 +388,79 @@ class ConnectionTests(helper.CPWebCase):
         self.status, self.headers, self.body = webtest.shb(response)
         self.assertStatus(200)
         self.assertBody("thanks for 'I am a small file' (text/plain)")
+        conn.close()
+    
+    def test_readall_or_close(self):
+        if cherrypy.server.protocol_version != "HTTP/1.1":
+            print "skipped ",
+            return
+        
+        self.PROTOCOL = "HTTP/1.1"
+        
+        if self.scheme == "https":
+            self.HTTP_CONN = httplib.HTTPSConnection
+        else:
+            self.HTTP_CONN = httplib.HTTPConnection
+        
+        # Test a max of 0 (the default) and then reset to what it was above.
+        old_max = cherrypy.server.max_request_body_size
+        for new_max in (0, old_max):
+            cherrypy.server.max_request_body_size = new_max
+            
+            self.persistent = True
+            conn = self.HTTP_CONN
+            
+            # Get a POST page with an error
+            conn.putrequest("POST", "/err_before_read", skip_host=True)
+            conn.putheader("Host", self.HOST)
+            conn.putheader("Content-Type", "text/plain")
+            conn.putheader("Content-Length", "1000")
+            conn.putheader("Expect", "100-continue")
+            conn.endheaders()
+            response = conn.response_class(conn.sock, method="POST")
+            
+            # ...assert and then skip the 100 response
+            version, status, reason = response._read_status()
+            self.assertEqual(status, 100)
+            while True:
+                skip = response.fp.readline().strip()
+                if not skip:
+                    break
+            
+            # ...send the body
+            conn.send("x" * 1000)
+            
+            # ...get the final response
+            response.begin()
+            self.status, self.headers, self.body = webtest.shb(response)
+            self.assertStatus(500)
+            
+            # Now try a working page with an Expect header...
+            conn._output('POST /upload HTTP/1.1')
+            conn._output("Host: %s" % self.HOST)
+            conn._output("Content-Type: text/plain")
+            conn._output("Content-Length: 17")
+            conn._output("Expect: 100-continue")
+            conn._send_output()
+            response = conn.response_class(conn.sock, method="POST")
+            
+            # ...assert and then skip the 100 response
+            version, status, reason = response._read_status()
+            self.assertEqual(status, 100)
+            while True:
+                skip = response.fp.readline().strip()
+                if not skip:
+                    break
+            
+            # ...send the body
+            conn.send("I am a small file")
+            
+            # ...get the final response
+            response.begin()
+            self.status, self.headers, self.body = webtest.shb(response)
+            self.assertStatus(200)
+            self.assertBody("thanks for 'I am a small file' (text/plain)")
+            conn.close()
     
     def test_No_Message_Body(self):
         if cherrypy.server.protocol_version != "HTTP/1.1":
@@ -353,7 +471,6 @@ class ConnectionTests(helper.CPWebCase):
         
         # Set our HTTP_CONN to an instance so it persists between requests.
         self.persistent = True
-        conn = self.HTTP_CONN
         
         # Make the first request and assert there's no "Connection: close".
         self.getPage("/")
@@ -412,11 +529,12 @@ class ConnectionTests(helper.CPWebCase):
         
         # Try a chunked request that exceeds server.max_request_body_size.
         # Note that the delimiters and trailer are included.
-        body = "5f\r\n" + ("x" * 95) + "\r\n0\r\n\r\n"
+        body = "3e3\r\n" + ("x" * 995) + "\r\n0\r\n\r\n"
         conn.putrequest("POST", "/upload", skip_host=True)
         conn.putheader("Host", self.HOST)
         conn.putheader("Transfer-Encoding", "chunked")
         conn.putheader("Content-Type", "text/plain")
+        # Chunked requests don't need a content-length
 ##        conn.putheader("Content-Length", len(body))
         conn.endheaders()
         conn.send(body)
@@ -424,6 +542,23 @@ class ConnectionTests(helper.CPWebCase):
         self.status, self.headers, self.body = webtest.shb(response)
         self.assertStatus(413)
         self.assertBody("")
+        conn.close()
+    
+    def test_Content_Length(self):
+        # Try a non-chunked request where Content-Length exceeds
+        # server.max_request_body_size. Assert error before body send.
+        self.persistent = True
+        conn = self.HTTP_CONN
+        conn.putrequest("POST", "/upload", skip_host=True)
+        conn.putheader("Host", self.HOST)
+        conn.putheader("Content-Type", "text/plain")
+        conn.putheader("Content-Length", 9999)
+        conn.endheaders()
+        response = conn.getresponse()
+        self.status, self.headers, self.body = webtest.shb(response)
+        self.assertStatus(413)
+        self.assertBody("")
+        conn.close()
     
     def test_HTTP10(self):
         self.PROTOCOL = "HTTP/1.0"
@@ -453,8 +588,28 @@ class ConnectionTests(helper.CPWebCase):
         self.assertBody(pov)
         # Apache, for example, may emit a Connection header even for HTTP/1.0
 ##        self.assertNoHeader("Connection")
+    
+    def test_598(self):
+        remote_data_conn = urllib.urlopen('%s://%s:%s/one_megabyte_of_a/' %
+                                          (self.scheme, self.HOST, self.PORT,))
+        buf = remote_data_conn.read(512)
+        time.sleep(timeout * 0.6)
+        remaining = (1024 * 1024) - 512
+        while remaining:
+            data = remote_data_conn.read(remaining)
+            if not data:
+                break
+            else:
+                buf += data
+            remaining -= len(data)
+       
+        self.assertEqual(len(buf), 1024 * 1024)
+        self.assertEqual(buf, "a" * 1024 * 1024)
+        self.assertEqual(remaining, 0)
+        remote_data_conn.close()
 
 
 if __name__ == "__main__":
     setup_server()
-    helper.testmain()
+    helper.testmain({'server.socket_timeout': timeout,
+                     'server.socket_host': '127.0.0.1'})
