@@ -37,7 +37,5 @@ cherrypy.tree.mount(GeneratorDemo())
 
 if __name__ == '__main__':
     import os.path
-    cherrypy.config.update(os.path.join(os.path.dirname(__file__), 'tutorial.conf'))
-    cherrypy.server.quickstart()
-    cherrypy.engine.start()
-
+    thisdir = os.path.dirname(__file__)
+    cherrypy.quickstart(config=os.path.join(thisdir, 'tutorial.conf'))
